@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2003, 2004, 2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2003-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Nicolas Delon <nicolas.delon@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -31,6 +31,7 @@
 #include "prelude-error.h"
 #include "prelude-inttypes.h"
 #include "prelude-string.h"
+#include "common.h"
 #include "idmef-data.h"
 
 
@@ -45,31 +46,32 @@
 #define IDMEF_DATA_OWN_DATA       0x2
 
 
-#define IDMEF_DATA_DECL(idmef_data_type, c_type, name)		\
-int idmef_data_new_ ## name(idmef_data_t **nd, c_type val)     	\
-{								\
-        int ret;              					\
-								\
-        ret = idmef_data_new(nd);	       			\
-        if ( ret < 0 )						\
-                return ret;					\
-								\
-        idmef_data_set_ ## name(*nd, val);			\
-								\
-        return ret;						\
-}								\
-								\
-void idmef_data_set_ ## name(idmef_data_t *ptr, c_type val)	\
-{								\
-        idmef_data_destroy_internal(ptr);			\
-        ptr->type = idmef_data_type;				\
-        ptr->len = sizeof(val);					\
-	ptr->data.name ## _data = val;				\
-}								\
-								\
-c_type idmef_data_get_ ## name(const idmef_data_t *ptr)		\
-{								\
-	return ptr->data.name ## _data;				\
+#define IDMEF_DATA_DECL(idmef_data_type, c_type, name)                \
+int idmef_data_new_ ## name(idmef_data_t **nd, c_type val)            \
+{                                                                     \
+        int ret;                                                      \
+                                                                      \
+        ret = idmef_data_new(nd);                                     \
+        if ( ret < 0 )                                                \
+                return ret;                                           \
+                                                                      \
+        idmef_data_set_ ## name(*nd, val);                            \
+                                                                      \
+        return ret;                                                   \
+}                                                                     \
+                                                                      \
+void idmef_data_set_ ## name(idmef_data_t *ptr, c_type val)           \
+{                                                                     \
+        prelude_return_if_fail(ptr);                                  \
+        idmef_data_destroy_internal(ptr);                             \
+        ptr->type = idmef_data_type;                                  \
+        ptr->len = sizeof(val);                                       \
+        ptr->data.name ## _data = val;                                \
+}                                                                     \
+                                                                      \
+c_type idmef_data_get_ ## name(const idmef_data_t *ptr)               \
+{                                                                     \
+        return ptr->data.name ## _data;                               \
 }
 
 
@@ -96,6 +98,8 @@ int idmef_data_new(idmef_data_t **data)
 
 idmef_data_t *idmef_data_ref(idmef_data_t *data)
 {
+        prelude_return_val_if_fail(data, NULL);
+
         data->refcount++;
         return data;
 }
@@ -104,49 +108,55 @@ idmef_data_t *idmef_data_ref(idmef_data_t *data)
 
 int idmef_data_set_ptr_ref_fast(idmef_data_t *data, idmef_data_type_t type, const void *ptr, size_t len)
 {
-	idmef_data_destroy_internal(data);
+        prelude_return_val_if_fail(data && ptr, -1);
 
-	data->type = type;
-	data->data.ro_data = ptr;
-	data->len = len;
-	
-	return 0;
+        idmef_data_destroy_internal(data);
+
+        data->type = type;
+        data->data.ro_data = ptr;
+        data->len = len;
+
+        return 0;
 }
 
 
 
 int idmef_data_set_ptr_dup_fast(idmef_data_t *data, idmef_data_type_t type, const void *ptr, size_t len)
 {
-	void *new;
+        void *new;
 
-	idmef_data_destroy_internal(data);
+        prelude_return_val_if_fail(data && ptr, -1);
 
-	new = malloc(len);
-	if ( ! new )
-		return -1;
+        idmef_data_destroy_internal(data);
 
-	memcpy(new, ptr, len);
+        new = malloc(len);
+        if ( ! new )
+                return -1;
 
-	data->type = type;
-	data->data.rw_data = new;
-	data->len = len;
-	data->flags |= IDMEF_DATA_OWN_DATA;
+        memcpy(new, ptr, len);
 
-	return 0;
+        data->type = type;
+        data->data.rw_data = new;
+        data->len = len;
+        data->flags |= IDMEF_DATA_OWN_DATA;
+
+        return 0;
 }
 
 
 
 int idmef_data_set_ptr_nodup_fast(idmef_data_t *data, idmef_data_type_t type, void *ptr, size_t len)
 {
-	idmef_data_destroy_internal(data);
+        prelude_return_val_if_fail(data && ptr, -1);
 
-	data->type = type;
-	data->data.rw_data= ptr;
-	data->len = len;
-	data->flags |= IDMEF_DATA_OWN_DATA;
+        idmef_data_destroy_internal(data);
 
-	return 0;
+        data->type = type;
+        data->data.rw_data= ptr;
+        data->len = len;
+        data->flags |= IDMEF_DATA_OWN_DATA;
+
+        return 0;
 }
 
 
@@ -154,16 +164,16 @@ int idmef_data_set_ptr_nodup_fast(idmef_data_t *data, idmef_data_type_t type, vo
 int idmef_data_new_ptr_ref_fast(idmef_data_t **data, idmef_data_type_t type, const void *ptr, size_t len)
 {
         int ret;
-        
-	ret = idmef_data_new(data);
-	if ( ret < 0 )
-		return ret;
+
+        ret = idmef_data_new(data);
+        if ( ret < 0 )
+                return ret;
 
         ret = idmef_data_set_ptr_ref_fast(*data, type, ptr, len);
         if ( ret < 0 )
-		idmef_data_destroy(*data);
+                idmef_data_destroy(*data);
 
-	return ret;
+        return ret;
 }
 
 
@@ -173,12 +183,12 @@ int idmef_data_new_ptr_dup_fast(idmef_data_t **data, idmef_data_type_t type, con
         int ret;
 
         ret = idmef_data_new(data);
-	if ( ret < 0 )
-		return ret;
+        if ( ret < 0 )
+                return ret;
 
         ret = idmef_data_set_ptr_dup_fast(*data, type, ptr, len);
         if ( ret < 0 )
-		idmef_data_destroy(*data);
+                idmef_data_destroy(*data);
 
         return ret;
 }
@@ -188,14 +198,14 @@ int idmef_data_new_ptr_dup_fast(idmef_data_t **data, idmef_data_type_t type, con
 int idmef_data_new_ptr_nodup_fast(idmef_data_t **data, idmef_data_type_t type, void *ptr, size_t len)
 {
         int ret;
-        
+
         ret = idmef_data_new(data);
-	if ( ret < 0 )
-		return ret;
+        if ( ret < 0 )
+                return ret;
 
         ret = idmef_data_set_ptr_nodup_fast(*data, type, ptr, len);
         if ( ret < 0 )
-		idmef_data_destroy(*data);
+                idmef_data_destroy(*data);
 
         return ret;
 }
@@ -214,9 +224,11 @@ int idmef_data_new_ptr_nodup_fast(idmef_data_t **data, idmef_data_type_t type, v
  */
 int idmef_data_copy_ref(const idmef_data_t *src, idmef_data_t *dst)
 {
+        prelude_return_val_if_fail(src && dst, -1);
+
         idmef_data_destroy_internal(dst);
 
-	dst->type = src->type;
+        dst->type = src->type;
         dst->len = src->len;
         dst->data = src->data;
         dst->flags &= ~IDMEF_DATA_OWN_DATA;
@@ -239,21 +251,23 @@ int idmef_data_copy_ref(const idmef_data_t *src, idmef_data_t *dst)
  */
 int idmef_data_copy_dup(const idmef_data_t *src, idmef_data_t *dst)
 {
+        prelude_return_val_if_fail(src && dst, -1);
+
         idmef_data_destroy_internal(dst);
 
-	dst->type = src->type;
-	dst->flags |= IDMEF_DATA_OWN_DATA;
-	dst->len = src->len;
+        dst->type = src->type;
+        dst->flags |= IDMEF_DATA_OWN_DATA;
+        dst->len = src->len;
 
-	if ( src->type == IDMEF_DATA_TYPE_CHAR_STRING || src->type == IDMEF_DATA_TYPE_BYTE_STRING ) {
-		dst->data.rw_data = malloc(src->len);
-		if ( ! dst->data.rw_data )
-			return -1;
-                
-		memcpy(dst->data.rw_data, src->data.ro_data, src->len);
-	} else {
-		dst->data = src->data;
-	}
+        if ( src->type == IDMEF_DATA_TYPE_CHAR_STRING || src->type == IDMEF_DATA_TYPE_BYTE_STRING ) {
+                dst->data.rw_data = malloc(src->len);
+                if ( ! dst->data.rw_data )
+                        return -1;
+
+                memcpy(dst->data.rw_data, src->data.ro_data, src->len);
+        } else {
+                dst->data = src->data;
+        }
 
         return 0;
 }
@@ -264,29 +278,33 @@ int idmef_data_clone(const idmef_data_t *src, idmef_data_t **dst)
 {
         int ret;
 
+        prelude_return_val_if_fail(src, -1);
+
         ret = idmef_data_new(dst);
         if ( ret < 0 )
                 return ret;
 
         ret = idmef_data_copy_dup(src, *dst);
-	if ( ret < 0 )
+        if ( ret < 0 )
                 idmef_data_destroy(*dst);
-        
-	return ret;
+
+        return ret;
 }
 
 
 
 const char *idmef_data_get_char_string(const idmef_data_t *data)
 {
-	return data->data.ro_data;
+        prelude_return_val_if_fail(data, NULL);
+        return data->data.ro_data;
 }
 
 
 
 const unsigned char *idmef_data_get_byte_string(const idmef_data_t *data)
 {
-	return data->data.ro_data;
+        prelude_return_val_if_fail(data, NULL);
+        return data->data.ro_data;
 }
 
 
@@ -299,7 +317,8 @@ const unsigned char *idmef_data_get_byte_string(const idmef_data_t *data)
  */
 idmef_data_type_t idmef_data_get_type(const idmef_data_t *data)
 {
-	return data->type;
+        prelude_return_val_if_fail(data, -1);
+        return data->type;
 }
 
 
@@ -313,6 +332,7 @@ idmef_data_type_t idmef_data_get_type(const idmef_data_t *data)
  */
 size_t idmef_data_get_len(const idmef_data_t *data)
 {
+        prelude_return_val_if_fail(data, 0);
         return data->len;
 }
 
@@ -327,18 +347,20 @@ size_t idmef_data_get_len(const idmef_data_t *data)
  */
 const void *idmef_data_get_data(const idmef_data_t *data)
 {
-	switch ( data->type ) {
-	case IDMEF_DATA_TYPE_UNKNOWN:
-		return NULL;
+        prelude_return_val_if_fail(data, NULL);
 
-	case IDMEF_DATA_TYPE_CHAR_STRING: case IDMEF_DATA_TYPE_BYTE_STRING:
-		return data->data.ro_data;
+        switch ( data->type ) {
+        case IDMEF_DATA_TYPE_UNKNOWN:
+                return NULL;
 
-	default:
-		return &data->data;
-	}
+        case IDMEF_DATA_TYPE_CHAR_STRING: case IDMEF_DATA_TYPE_BYTE_STRING:
+                return data->data.ro_data;
 
-	return NULL;
+        default:
+                return &data->data;
+        }
+
+        return NULL;
 }
 
 
@@ -351,6 +373,7 @@ const void *idmef_data_get_data(const idmef_data_t *data)
  */
 prelude_bool_t idmef_data_is_empty(const idmef_data_t *data)
 {
+        prelude_return_val_if_fail(data, TRUE);
         return (data->len == 0) ? TRUE : FALSE;
 }
 
@@ -366,26 +389,26 @@ static int bytes_to_string(prelude_string_t *out, const unsigned char *src, size
                 ret = prelude_string_ncat(out, &b64tbl[src[0] >> 2], 1);
                 if ( ret < 0 )
                         return ret;
-                
+
                 c = b64tbl[((src[0] << 4) + ((--size) ? src[1] >> 4 : 0)) & 0x3f];
 
                 ret = prelude_string_ncat(out, &c, 1);
                 if ( ret < 0 )
                         return ret;
-                        
+
                 c = (size) ? b64tbl[((src[1] << 2) + ((--size) ? src[2] >> 6 : 0)) & 0x3f] : '=';
-                
+
                 ret = prelude_string_ncat(out, &c, 1);
                 if ( ret < 0 )
                         return ret;
-               
+
                 c = (size && size--) ? b64tbl[src[2] & 0x3f] : '=';
-                
+
                 ret = prelude_string_ncat(out, &c, 1);
                 if ( ret < 0 )
                         return ret;
-                
-                src += 3;        
+
+                src += 3;
         }
 
         return 0;
@@ -406,45 +429,47 @@ static int bytes_to_string(prelude_string_t *out, const unsigned char *src, size
  */
 int idmef_data_to_string(const idmef_data_t *data, prelude_string_t *out)
 {
-	int ret = 0;
+        int ret = 0;
 
-	switch ( data->type ) {
-	case IDMEF_DATA_TYPE_UNKNOWN:
-		return 0;
+        prelude_return_val_if_fail(data && out, -1);
 
-	case IDMEF_DATA_TYPE_CHAR:
-		ret = prelude_string_sprintf(out, "%c", data->data.char_data);
-		break;
+        switch ( data->type ) {
+        case IDMEF_DATA_TYPE_UNKNOWN:
+                return 0;
 
-	case IDMEF_DATA_TYPE_BYTE:
+        case IDMEF_DATA_TYPE_CHAR:
+                ret = prelude_string_sprintf(out, "%c", data->data.char_data);
+                break;
+
+        case IDMEF_DATA_TYPE_BYTE:
                 /*
                  * %hh convertion specifier is not portable.
                  */
-		ret = prelude_string_sprintf(out, "%u", (unsigned int) data->data.byte_data);
-		break;
+                ret = prelude_string_sprintf(out, "%u", (unsigned int) data->data.byte_data);
+                break;
 
-	case IDMEF_DATA_TYPE_UINT32:
-		ret = prelude_string_sprintf(out, "%u", data->data.uint32_data);
-		break;
+        case IDMEF_DATA_TYPE_UINT32:
+                ret = prelude_string_sprintf(out, "%u", data->data.uint32_data);
+                break;
 
-	case IDMEF_DATA_TYPE_UINT64:
-		ret = prelude_string_sprintf(out, "%" PRELUDE_PRIu64, data->data.uint64_data);
-		break;
+        case IDMEF_DATA_TYPE_UINT64:
+                ret = prelude_string_sprintf(out, "%" PRELUDE_PRIu64, data->data.uint64_data);
+                break;
 
-	case IDMEF_DATA_TYPE_FLOAT:
-		ret = prelude_string_sprintf(out, "%f", data->data.float_data);
-		break;
+        case IDMEF_DATA_TYPE_FLOAT:
+                ret = prelude_string_sprintf(out, "%f", data->data.float_data);
+                break;
 
-	case IDMEF_DATA_TYPE_CHAR_STRING:
-		ret = prelude_string_sprintf(out, "%s", (const char *) data->data.ro_data);
-		break;
+        case IDMEF_DATA_TYPE_CHAR_STRING:
+                ret = prelude_string_sprintf(out, "%s", (const char *) data->data.ro_data);
+                break;
 
-	case IDMEF_DATA_TYPE_BYTE_STRING:
-		ret = bytes_to_string(out, data->data.ro_data, data->len);
-		break;
-	}
+        case IDMEF_DATA_TYPE_BYTE_STRING:
+                ret = bytes_to_string(out, data->data.ro_data, data->len);
+                break;
+        }
 
-	return ret;
+        return ret;
 }
 
 
@@ -454,9 +479,11 @@ int idmef_data_to_string(const idmef_data_t *data, prelude_string_t *out)
  *  from idmef-tree-wrap.c
  */
 void idmef_data_destroy_internal(idmef_data_t *ptr)
-{        
+{
+        prelude_return_if_fail(ptr);
+
         if ( (ptr->type == IDMEF_DATA_TYPE_CHAR_STRING || ptr->type == IDMEF_DATA_TYPE_BYTE_STRING) &&
-	     ptr->flags & IDMEF_DATA_OWN_DATA ) {
+             ptr->flags & IDMEF_DATA_OWN_DATA ) {
                 free(ptr->data.rw_data);
                 ptr->data.rw_data = NULL;
         }
@@ -478,11 +505,13 @@ void idmef_data_destroy_internal(idmef_data_t *ptr)
  */
 void idmef_data_destroy(idmef_data_t *data)
 {
+        prelude_return_if_fail(data);
+
         if ( --data->refcount )
                 return;
-        
+
         idmef_data_destroy_internal(data);
-        
+
         if ( data->flags & IDMEF_DATA_OWN_STRUCTURE )
                 free(data);
 }
@@ -521,31 +550,37 @@ int idmef_data_set_char_string_nodup_fast(idmef_data_t *data, char *ptr, size_t
 
 int idmef_data_new_char_string_ref(idmef_data_t **data, const char *ptr)
 {
+        prelude_return_val_if_fail(ptr, -1);
         return idmef_data_new_char_string_ref_fast(data, ptr, strlen(ptr));
 }
 
 int idmef_data_new_char_string_dup(idmef_data_t **data, const char *ptr)
 {
+        prelude_return_val_if_fail(ptr, -1);
         return idmef_data_new_char_string_dup_fast(data, ptr, strlen(ptr));
 }
 
 int idmef_data_new_char_string_nodup(idmef_data_t **data, char *ptr)
 {
+        prelude_return_val_if_fail(ptr, -1);
         return idmef_data_new_char_string_nodup_fast(data, ptr, strlen(ptr));
 }
 
 int idmef_data_set_char_string_ref(idmef_data_t *data, const char *ptr)
 {
+        prelude_return_val_if_fail(data && ptr, -1);
         return idmef_data_set_char_string_ref_fast(data, ptr, strlen(ptr));
 }
 
 int idmef_data_set_char_string_dup(idmef_data_t *data, const char *ptr)
 {
+        prelude_return_val_if_fail(data && ptr, -1);
         return idmef_data_set_char_string_dup_fast(data, ptr, strlen(ptr));
 }
 
 int idmef_data_set_char_string_nodup(idmef_data_t *data, char *ptr)
 {
+        prelude_return_val_if_fail(data && ptr, -1);
         return idmef_data_set_char_string_nodup_fast(data, ptr, strlen(ptr));
 }
 
@@ -555,37 +590,37 @@ int idmef_data_set_char_string_nodup(idmef_data_t *data, char *ptr)
  */
 int idmef_data_new_byte_string_ref(idmef_data_t **data, const unsigned char *ptr, size_t len)
 {
-	return idmef_data_new_ptr_ref_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
+        return idmef_data_new_ptr_ref_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
 }
 
 
 int idmef_data_new_byte_string_dup(idmef_data_t **data, const unsigned char *ptr, size_t len)
 {
-	return idmef_data_new_ptr_dup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
+        return idmef_data_new_ptr_dup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
 }
 
 
 int idmef_data_new_byte_string_nodup(idmef_data_t **data, unsigned char *ptr, size_t len)
 {
-	return idmef_data_new_ptr_nodup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
+        return idmef_data_new_ptr_nodup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
 }
 
 
 int idmef_data_set_byte_string_ref(idmef_data_t *data, const unsigned char *ptr, size_t len)
 {
-	return idmef_data_set_ptr_ref_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
+        return idmef_data_set_ptr_ref_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
 }
 
 
 int idmef_data_set_byte_string_dup(idmef_data_t *data, const unsigned char *ptr, size_t len)
 {
-	return idmef_data_set_ptr_dup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
+        return idmef_data_set_ptr_dup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
 }
 
 
 int idmef_data_set_byte_string_nodup(idmef_data_t *data, unsigned char *ptr, size_t len)
 {
-	return idmef_data_set_ptr_nodup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
+        return idmef_data_set_ptr_nodup_fast(data, IDMEF_DATA_TYPE_BYTE_STRING, ptr, len);
 }
 
 
@@ -597,10 +632,10 @@ int idmef_data_compare(const idmef_data_t *data1, const idmef_data_t *data2)
 
         else if ( ! data1 || ! data2 )
                 return (data1) ? 1 : -1;
-        
+
         else if ( data1->len != data2->len )
                 return (data1->len > data2->len) ? 1 : -1;
-        
+
         else if ( data1->type != data2->type )
                 return -1;
 
