@@ -32,21 +32,18 @@
 #include "config.h"
 
 #include <poll.h>
-#include <sys/types.h>
+#include <stdio.h>
+#include <string.h>
 #include <sys/socket.h>
+#include <sys/time.h>
+#include <sys/types.h>
+#include <time.h>
 #include "ftw_.h"
 #include "getaddrinfo.h"
+#include "getpass.h"
 #include "inet_ntop.h"
 #include "minmax.h"
 #include "pathmax.h"
 #include "regex.h"
-#include "snprintf.h"
-#include "strcase.h"
-#include "strcasestr.h"
-#include "strdup.h"
-#include "strndup.h"
-#include "strsep.h"
-#include "timegm.h"
-#include "vsnprintf.h"
 
 #endif /* _LIBPRELUDE_LIBMISSING_H */
