@@ -1,6 +1,6 @@
 /*****
 *
-* Copyright (C) 2004,2005 PreludeIDS Technologies. All Rights Reserved.
+* Copyright (C) 2004-2005,2006,2007 PreludeIDS Technologies. All Rights Reserved.
 * Author: Yoann Vandoorselaere <yoann.v@prelude-ids.com>
 *
 * This file is part of the Prelude library.
@@ -93,7 +93,7 @@ inline static int check_string_f(const char *f, int l, const char *str, size_t l
                 prelude_log(PRELUDE_LOG_WARN, "%s:%d: warning, wrap around detected.\n", f, l);
                 return prelude_error_verbose(PRELUDE_ERROR_INVAL_LENGTH, "string warning: wrap around would occur");
         }
-        
+
         if ( str[len] != 0 ) {
                 prelude_log(PRELUDE_LOG_WARN, "%s:%d: warning, string is not NULL terminated.\n", f, l);
                 return prelude_error_verbose(PRELUDE_ERROR_STRING_NOT_NULL_TERMINATED, "string warning: not nul terminated");
@@ -109,12 +109,12 @@ static int string_buf_alloc(prelude_string_t *string, size_t len)
          * include room for terminating \0.
          */
         string->data.rwbuf = malloc(len + 1);
-        if ( ! string->data.rwbuf ) 
+        if ( ! string->data.rwbuf )
                 return prelude_error_from_errno(errno);
-        
+
         string->index = len;
         string->size = len + 1;
-        
+
         return 0;
 }
 
@@ -123,7 +123,7 @@ static int string_buf_alloc(prelude_string_t *string, size_t len)
 static void string_buf_copy(prelude_string_t *string, const char *buf, size_t len)
 {
         assert(len < string->size);
-        
+
         memcpy(string->data.rwbuf, buf, len);
         string->data.rwbuf[len] = '\0';
 }
@@ -134,7 +134,7 @@ static int allocate_more_chunk_if_needed(prelude_string_t *s, size_t needed_len)
 {
         char *ptr;
         size_t len;
-        
+
         if ( needed_len )
                 len = MAX(needed_len - (s->size - s->index), CHUNK_SIZE);
         else
@@ -142,29 +142,29 @@ static int allocate_more_chunk_if_needed(prelude_string_t *s, size_t needed_len)
 
         if ( s->size + len < s->size )
                 return prelude_error(PRELUDE_ERROR_INVAL_LENGTH);
-                
+
         if ( s->flags & PRELUDE_STRING_CAN_REALLOC ) {
-                
+
                 ptr = _prelude_realloc(s->data.rwbuf, s->size + len);
                 if ( ! ptr )
                         return prelude_error_from_errno(errno);
         }
 
-        else {             
+        else {
                 ptr = malloc(s->size + len);
                 if ( ! ptr )
                         return prelude_error_from_errno(errno);
-                
+
                 if ( s->data.robuf )
                         memcpy(ptr, s->data.robuf, s->index + 1);
-                
+
                 s->flags |= PRELUDE_STRING_CAN_REALLOC|PRELUDE_STRING_OWN_DATA;
         }
 
         s->size += len;
         s->data.rwbuf = ptr;
 
-	return 0;
+        return 0;
 }
 
 
@@ -176,17 +176,17 @@ static int allocate_more_chunk_if_needed(prelude_string_t *s, size_t needed_len)
  * Create a new #prelude_string_t object, and store in in @string.
  *
  * Returns: 0 on success, or a negative value if an error occured.
- */        
+ */
 int prelude_string_new(prelude_string_t **string)
 {
         *string = calloc(1, sizeof(**string));
         if ( ! *string )
                 return prelude_error_from_errno(errno);
-        
+
         (*string)->refcount = 1;
         prelude_list_init(&(*string)->list);
         (*string)->flags = PRELUDE_STRING_OWN_STRUCTURE;
-        
+
         return 0;
 }
 
@@ -201,10 +201,12 @@ int prelude_string_new(prelude_string_t **string)
  *
  * Returns: @string.
  */
-prelude_string_t *prelude_string_ref(prelude_string_t *data)
+prelude_string_t *prelude_string_ref(prelude_string_t *string)
 {
-        data->refcount++;
-        return data;
+        prelude_return_val_if_fail(string, NULL);
+
+        string->refcount++;
+        return string;
 }
 
 
@@ -224,11 +226,13 @@ prelude_string_t *prelude_string_ref(prelude_string_t *data)
 int prelude_string_new_dup_fast(prelude_string_t **string, const char *str, size_t len)
 {
         int ret;
-        
+
+        prelude_return_val_if_fail(str, -1);
+
         ret = check_string(str, len);
         if ( ret < 0 )
                 return ret;
-        
+
         ret = prelude_string_new(string);
         if ( ret < 0 )
                 return ret;
@@ -236,10 +240,10 @@ int prelude_string_new_dup_fast(prelude_string_t **string, const char *str, size
         ret = string_buf_alloc(*string, len);
         if ( ret < 0 )
                 return ret;
-        
+
         string_buf_copy(*string, str, len);
         (*string)->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
-                
+
         return 0;
 }
 
@@ -258,6 +262,7 @@ int prelude_string_new_dup_fast(prelude_string_t **string, const char *str, size
  */
 int prelude_string_new_dup(prelude_string_t **string, const char *str)
 {
+        prelude_return_val_if_fail(str, -1);
         return prelude_string_new_dup_fast(string, str, strlen(str));
 }
 
@@ -279,6 +284,8 @@ int prelude_string_new_nodup_fast(prelude_string_t **string, char *str, size_t l
 {
         int ret;
 
+        prelude_return_val_if_fail(str, -1);
+
         ret = check_string(str, len);
         if ( ret < 0 )
                 return ret;
@@ -291,7 +298,7 @@ int prelude_string_new_nodup_fast(prelude_string_t **string, char *str, size_t l
         (*string)->size = len + 1;
         (*string)->data.rwbuf = str;
         (*string)->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
-        
+
         return 0;
 }
 
@@ -310,6 +317,7 @@ int prelude_string_new_nodup_fast(prelude_string_t **string, char *str, size_t l
  */
 int prelude_string_new_nodup(prelude_string_t **string, char *str)
 {
+        prelude_return_val_if_fail(str, -1);
         return prelude_string_new_nodup_fast(string, str, strlen(str));
 }
 
@@ -330,10 +338,12 @@ int prelude_string_new_ref_fast(prelude_string_t **string, const char *buf, size
 {
         int ret;
 
+        prelude_return_val_if_fail(buf, -1);
+
         ret = check_string(buf, len);
         if ( ret < 0 )
                 return ret;
-        
+
         ret = prelude_string_new(string);
         if ( ret < 0 )
                 return ret;
@@ -341,7 +351,7 @@ int prelude_string_new_ref_fast(prelude_string_t **string, const char *buf, size
         (*string)->index = len;
         (*string)->size = len + 1;
         (*string)->data.robuf = buf;
-        
+
         return 0;
 }
 
@@ -359,6 +369,7 @@ int prelude_string_new_ref_fast(prelude_string_t **string, const char *buf, size
  */
 int prelude_string_new_ref(prelude_string_t **string, const char *str)
 {
+        prelude_return_val_if_fail(str, -1);
         return prelude_string_new_ref_fast(string, str, strlen(str));
 }
 
@@ -378,20 +389,22 @@ int prelude_string_new_ref(prelude_string_t **string, const char *str)
 int prelude_string_set_dup_fast(prelude_string_t *string, const char *buf, size_t len)
 {
         int ret;
-        
+
+        prelude_return_val_if_fail(string && buf, -1);
+
         ret = check_string(buf, len);
         if ( ret < 0 )
                 return ret;
-        
+
         prelude_string_destroy_internal(string);
 
         ret = string_buf_alloc(string, len);
         if ( ret < 0 )
                 return ret;
-        
+
         string_buf_copy(string, buf, len);
         string->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
-        
+
         return 0;
 }
 
@@ -409,6 +422,7 @@ int prelude_string_set_dup_fast(prelude_string_t *string, const char *buf, size_
  */
 int prelude_string_set_dup(prelude_string_t *string, const char *buf)
 {
+        prelude_return_val_if_fail(string && buf, -1);
         return prelude_string_set_dup_fast(string, buf, strlen(buf));
 }
 
@@ -429,19 +443,21 @@ int prelude_string_set_nodup_fast(prelude_string_t *string, char *buf, size_t le
 {
         int ret;
 
+        prelude_return_val_if_fail(string && buf, -1);
+
         ret = check_string(buf, len);
         if ( ret < 0 )
                 return ret;
-                
+
         prelude_string_destroy_internal(string);
 
         string->index = len;
         string->size = len + 1;
         string->data.rwbuf = buf;
-        
+
         string->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
 
-        return 0;       
+        return 0;
 }
 
 
@@ -459,6 +475,7 @@ int prelude_string_set_nodup_fast(prelude_string_t *string, char *buf, size_t le
  */
 int prelude_string_set_nodup(prelude_string_t *string, char *buf)
 {
+        prelude_return_val_if_fail(string && buf, -1);
         return prelude_string_set_nodup_fast(string, buf, strlen(buf));
 }
 
@@ -479,16 +496,18 @@ int prelude_string_set_ref_fast(prelude_string_t *string, const char *buf, size_
 {
         int ret;
 
+        prelude_return_val_if_fail(string && buf, -1);
+
         ret = check_string(buf, len);
         if ( ret < 0 )
                 return ret;
-                
+
         prelude_string_destroy_internal(string);
 
         string->index = len;
         string->size = len + 1;
         string->data.robuf = buf;
- 
+
         string->flags &= ~(PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC);
 
         return 0;
@@ -508,6 +527,7 @@ int prelude_string_set_ref_fast(prelude_string_t *string, const char *buf, size_
  */
 int prelude_string_set_ref(prelude_string_t *string, const char *buf)
 {
+        prelude_return_val_if_fail(string && buf, -1);
         return prelude_string_set_ref_fast(string, buf, strlen(buf));
 }
 
@@ -525,14 +545,16 @@ int prelude_string_set_ref(prelude_string_t *string, const char *buf)
  * Returns: 0 on success, or a negative value if an error occured.
  */
 int prelude_string_copy_ref(const prelude_string_t *src, prelude_string_t *dst)
-{        
+{
+        prelude_return_val_if_fail(src && dst, -1);
+
         prelude_string_destroy_internal(dst);
 
         dst->size = src->size;
         dst->index = src->index;
         dst->data.robuf = src->data.robuf;
         dst->flags &= ~(PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC);
-                
+
         return 0;
 }
 
@@ -550,18 +572,20 @@ int prelude_string_copy_ref(const prelude_string_t *src, prelude_string_t *dst)
  */
 int prelude_string_copy_dup(const prelude_string_t *src, prelude_string_t *dst)
 {
+        prelude_return_val_if_fail(src && dst, -1);
+
         prelude_string_destroy_internal(dst);
 
         dst->data.rwbuf = malloc(src->size);
         if ( ! dst->data.rwbuf )
                 return prelude_error_from_errno(errno);
-        
+
         dst->size = src->size;
         dst->index = src->index;
         dst->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
-        
+
         string_buf_copy(dst, src->data.robuf, src->index);
-        
+
         return 0;
 }
 
@@ -580,11 +604,13 @@ int prelude_string_copy_dup(const prelude_string_t *src, prelude_string_t *dst)
 int prelude_string_clone(const prelude_string_t *src, prelude_string_t **dst)
 {
         int ret;
-        
+
+        prelude_return_val_if_fail(src, -1);
+
         ret = prelude_string_new(dst);
         if ( ret < 0 )
                 return ret;
-        
+
         (*dst)->data.rwbuf = malloc(src->size);
         if ( ! (*dst)->data.rwbuf ) {
                 free(*dst);
@@ -596,7 +622,7 @@ int prelude_string_clone(const prelude_string_t *src, prelude_string_t **dst)
         (*dst)->flags |= PRELUDE_STRING_OWN_DATA|PRELUDE_STRING_CAN_REALLOC;
 
         string_buf_copy(*dst, src->data.robuf, src->index);
-        
+
         return 0;
 }
 
@@ -612,6 +638,7 @@ int prelude_string_clone(const prelude_string_t *src, prelude_string_t **dst)
  */
 size_t prelude_string_get_len(const prelude_string_t *string)
 {
+        prelude_return_val_if_fail(string, 0);
         return string->index;
 }
 
@@ -630,6 +657,7 @@ size_t prelude_string_get_len(const prelude_string_t *string)
  */
 const char *prelude_string_get_string_or_default(const prelude_string_t *string, const char *def)
 {
+        prelude_return_val_if_fail(string, NULL);
         return string->data.robuf ? string->data.robuf : def;
 }
 
@@ -647,7 +675,8 @@ const char *prelude_string_get_string_or_default(const prelude_string_t *string,
  */
 const char *prelude_string_get_string(const prelude_string_t *string)
 {
-        return string ? string->data.robuf : NULL;
+        prelude_return_val_if_fail(string, NULL);
+        return string->data.robuf;
 }
 
 
@@ -664,9 +693,10 @@ const char *prelude_string_get_string(const prelude_string_t *string)
  * Returns: 0 on success, or a negative value if an error occured.
  */
 int prelude_string_get_string_released(prelude_string_t *string, char **outptr)
-{        
+{
+        prelude_return_val_if_fail(string, -1);
         *outptr = NULL;
-        
+
         if ( ! string->index )
                 return 0;
 
@@ -677,7 +707,7 @@ int prelude_string_get_string_released(prelude_string_t *string, char **outptr)
 
         if ( string->index + 1 <= string->index )
                 return prelude_error(PRELUDE_ERROR_INVAL_LENGTH);
-        
+
         *outptr = _prelude_realloc(string->data.rwbuf, string->index + 1);
         if ( ! *outptr )
                 return prelude_error_from_errno(errno);
@@ -685,7 +715,7 @@ int prelude_string_get_string_released(prelude_string_t *string, char **outptr)
         string->size = 0;
         string->index = 0;
         string->data.rwbuf = NULL;
-        
+
         return 0;
 }
 
@@ -702,6 +732,7 @@ int prelude_string_get_string_released(prelude_string_t *string, char **outptr)
  */
 prelude_bool_t prelude_string_is_empty(const prelude_string_t *string)
 {
+        prelude_return_val_if_fail(string, TRUE);
         return (string->index == 0) ? TRUE : FALSE;
 }
 
@@ -712,7 +743,9 @@ prelude_bool_t prelude_string_is_empty(const prelude_string_t *string)
  *  from idmef-tree-wrap.c
  */
 void prelude_string_destroy_internal(prelude_string_t *string)
-{        
+{
+        prelude_return_if_fail(string);
+
         if ( (string->flags & PRELUDE_STRING_OWN_DATA) && string->data.rwbuf ) {
                 free(string->data.rwbuf);
                 string->data.rwbuf = NULL;
@@ -736,15 +769,17 @@ void prelude_string_destroy_internal(prelude_string_t *string)
  * or a referenced string that have been modified.
  */
 void prelude_string_destroy(prelude_string_t *string)
-{        
+{
+        prelude_return_if_fail(string);
+
         if ( --string->refcount )
                 return;
-        
+
         if ( ! prelude_list_is_empty(&string->list) )
                 prelude_list_del_init(&string->list);
-        
+
         prelude_string_destroy_internal(string);
-        
+
         if ( string->flags & PRELUDE_STRING_OWN_STRUCTURE )
                 free(string);
 }
@@ -768,7 +803,9 @@ int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap
 {
         int ret;
         va_list bkp;
-        
+
+        prelude_return_val_if_fail(string && fmt, -1);
+
         if ( ! (string->flags & PRELUDE_STRING_CAN_REALLOC) ) {
 
                 ret = allocate_more_chunk_if_needed(string, 0);
@@ -778,7 +815,7 @@ int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap
 
         PRELUDE_VA_COPY(bkp, ap);
         ret = vsnprintf(string->data.rwbuf + string->index, string->size - string->index, fmt, ap);
-        
+
         /*
          * From sprintf(3) on GNU/Linux:
          *
@@ -798,7 +835,7 @@ int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap
         ret = allocate_more_chunk_if_needed(string, (ret < 0) ? 0 : ret + 1);
         if ( ret < 0 )
                 goto end;
-        
+
         ret = prelude_string_vprintf(string, fmt, bkp);
 
  end:
@@ -822,14 +859,16 @@ int prelude_string_vprintf(prelude_string_t *string, const char *fmt, va_list ap
  */
 int prelude_string_sprintf(prelude_string_t *string, const char *fmt, ...)
 {
-	int ret;
-	va_list ap;
-	
+        int ret;
+        va_list ap;
+
+        prelude_return_val_if_fail(string && fmt, -1);
+
         va_start(ap, fmt);
         ret = prelude_string_vprintf(string, fmt, ap);
         va_end(ap);
-                		
-	return ret;
+
+        return ret;
 }
 
 
@@ -850,24 +889,26 @@ int prelude_string_ncat(prelude_string_t *dst, const char *str, size_t len)
 {
         int ret;
 
-	if ( dst->flags & PRELUDE_STRING_CAN_REALLOC && len < (dst->size - dst->index) ) {
-                
+        prelude_return_val_if_fail(dst && str, -1);
+
+        if ( dst->flags & PRELUDE_STRING_CAN_REALLOC && len < (dst->size - dst->index) ) {
+
                 memcpy(dst->data.rwbuf + dst->index, str, len);
 
                 dst->index += len;
                 dst->data.rwbuf[dst->index] = '\0';
 
-		return len;
-	}
+                return len;
+        }
 
         if ( len + 1 < len )
                 return prelude_error(PRELUDE_ERROR_INVAL_LENGTH);
-        
+
         ret = allocate_more_chunk_if_needed(dst, len + 1);
-	if ( ret < 0 )
+        if ( ret < 0 )
                 return ret;
 
-	return prelude_string_ncat(dst, str, len);
+        return prelude_string_ncat(dst, str, len);
 }
 
 
@@ -879,13 +920,14 @@ int prelude_string_ncat(prelude_string_t *dst, const char *str, size_t len)
  *
  * The prelude_string_cat() function appends the @str string to the @dst
  * prelude_string_t object over-writing the `\0' character at the end of
- * @dst, and then adds a termi-nating `\0' character. 
+ * @dst, and then adds a termi-nating `\0' character.
  *
  * Returns: @len, or a negative value if an error occured.
  */
 int prelude_string_cat(prelude_string_t *dst, const char *str)
 {
-	return prelude_string_ncat(dst, str, strlen(str));
+        prelude_return_val_if_fail(dst && str, -1);
+        return prelude_string_ncat(dst, str, strlen(str));
 }
 
 
@@ -899,8 +941,10 @@ int prelude_string_cat(prelude_string_t *dst, const char *str)
  */
 void prelude_string_clear(prelude_string_t *string)
 {
+        prelude_return_if_fail(string);
+
         string->index = 0;
-        
+
         if ( string->data.rwbuf && string->flags & PRELUDE_STRING_OWN_DATA )
                 *(string->data.rwbuf) = '\0';
 
@@ -923,12 +967,12 @@ int prelude_string_compare(const prelude_string_t *str1, const prelude_string_t
 
         else if ( ! str1 || ! str2 )
                 return -1;
-        
+
         else if ( str1->index != str2->index )
                 return -1;
 
         if ( str1->index == 0 )
                 return 0;
-        
+
         return strcmp(str1->data.robuf, str2->data.robuf);
 }
